/*
|| Create package bodies
*/
CREATE OR REPLACE package body forum is

   /*
   || Form to post a response to a message
   */
   procedure create_msg_form ( i_parent_msg in varchar2 default null) is
   begin
      -- Only allow users that are logged in to post
      if forum_users.get_current_user is null then
	     forum_users.login_form(null, 'You must login to post a message');
	  else
	     -- Print message form
         htp.print('<body bgcolor=white>');
         -- Print a header; a msg_id of zero indicates a forum topic
	     if i_parent_msg = 0 then
            htp.print('<h1>Create Forum Topic</h1><hr>');
	     else
            htp.print('<h1>Post response</h1><hr>');
         end if;
         htp.print('<form action=forum.save_message method=POST>');
	     htp.print('<table>');
         htp.formHidden(cname=>'i_parent_msg', cvalue=> i_parent_msg);
	     general_form.print_input_row('Subject', 'i_subject');
	     general_form.print_textarea_row ('Body', 'i_msg_body', 80,10);
	     htp.print('</table>');
         htp.formSubmit;
	     htp.print('</form>');
	  end if;
   end;

   /*
   || Procedure to save a message
   */
   procedure save_message (
      i_parent_msg in varchar2 default null,
	  i_subject in varchar2 default null,
	  i_msg_body in varchar2 default null ) is

	  author members.USERNAME%TYPE := forum_users.get_current_user;

   begin
      -- Only allow users that are logged in to post
      if forum_users.get_current_user is null then
	     forum_users.login_form(null, 'You must login to post a message');
	  else
         -- save message
   	     insert into messages
	        (msg_id, msg_parent, msg_author, msg_subject, msg_body )
	     values
		    (message_seq.nextval, i_parent_msg, author, i_subject, i_msg_body);
 	     commit;
	     -- Now return to the original message to display the new post
		 if i_parent_msg = 0 then
  	        current_forum_list;
		 else
		    view_message(i_parent_msg);
		 end if;
	  end if;
   exception
      when others then
         htp.print('<body bgcolor=white>');
         htp.print('<b>An error has occured<p>');
   end;

   /*
   || Print all messages for a thread
   */
   procedure print_thread_links (
      i_thread_id in varchar2,
	  i_expand_thread in varchar2 default 'N' ) is

	  cursor t_cur is
	     select * from messages where msg_parent = i_thread_id
		    order by date_created;

	  t_rec t_cur%ROWTYPE;
	  link varchar2(500);

   begin
      htp.print('<h4>');
      htp.print('<ol>');
      open t_cur;
	  loop
	     fetch t_cur into t_rec;
		 exit when t_cur%NOTFOUND;
		 htp.print('<li>');
	     link := 'forum.view_message?i_thread_id=' || t_rec.msg_id;
	   	 htp.anchor (link, t_rec.msg_subject);
		 htp.italic('(' || t_rec.msg_author || ',' || t_rec.date_created || ')');
		 htp.print('</li>');
		 -- Recursively print the children if the expand flag is ture
		 if i_expand_thread = 'Y' then
   		    print_thread_links(t_rec.msg_id, 'Y' );
		 end if;
	  end loop;
	  htp.print('</ol>');
      htp.print('</h4>');
	  close t_cur;
   end;

   /*
   || Print all the forums (msg_parent_id = 0) in order by date created
   */
   procedure current_forum_list is
	  link varchar2(200);
   begin
      htp.print('<body bgcolor=white>');
      htp.print('<h1>Forums</h1><br>');
      link := 'forum.create_msg_form?i_parent_msg=0';
      htp.anchor (link, 'Create a new forum');
	  htp.print('<hr>');
	  print_thread_links(0, 'N');
   end;


   /*
   || Procedure to view a message
   */
   procedure view_message (
      i_thread_id in varchar2 default null,
	  i_hierarchy_flag in varchar2 default null ) is

	  m_rec messages%ROWTYPE;
	  msg_found boolean;
	  link varchar2(500);
   begin
      select * into m_rec from messages where
	     msg_id = i_thread_id order by date_created;
      htp.print('<body bgcolor=white>');
      htp.print('<h2>' || m_rec.msg_subject || '</h2>');
      htp.print('<i>Posted by ' || m_rec.msg_author );
      htp.print(' on ' || m_rec.date_created || '</i><p>');
      htp.print('<h2>Message:</h2><p>' ||  m_rec.msg_body || '<p>');
      link := 'forum.create_msg_form?i_parent_msg=' || i_thread_id ;
      htp.anchor (link, 'Respond');
	  htp.print('<h2>Previous Responses:</h2><p>');
      print_thread_links (i_thread_id, 'Y');
	  htp.print('<p>');
	  if m_rec.msg_parent != 0 then
         link := 'forum.view_message?i_thread_id=' || m_rec.msg_parent ;
         htp.anchor (link, 'Previous Message');
	  end if;
   exception
      when NO_DATA_FOUND then
	     htp.print('<h1>Message Not Found!</h1>');
   end;

end; 
/
/*--------------------------------------------------------------------------*/
CREATE OR REPLACE package body forum_members is


   /*
   || Print a single two-column table row; column 1 has a description,
   || column 2 has either an input element plain text
   */
   procedure print_form_row (
      i_label in varchar2,
	  i_input_name in varchar2,
	  i_hidden_flag in boolean default false ) is
   begin
      htp.print('<tr>');
	  htp.print('<th align=right>' || i_label ||':</th>');
	  htp.print('<td>');
      if not i_hidden_flag then
         htp.formText ( cname => i_input_name );
	  else
         htp.formPassword ( cname => i_input_name );
	  end if;
	  htp.print('</td>');
	  htp.print('</tr>');
   end;

   procedure create_user_form is
   begin
      htp.print('<h1>Enrolling New User</h1><hr>');
      htp.print('<form action=save_user_info>');
      -- Existing users cannot change their name or username
	  htp.print('<table>');
	  print_form_row('Forum User Name', 'i_username');
	  print_form_row('Real Name', 'i_name');
	  print_form_row('Forum Password', 'i_password', true);
	  print_form_row('Email Address', 'i_email_address');
	  -- The description is a multi-line, not a simepl text input
	  htp.print('<tr><th align=right valign=top>Description:</th>');
	  htp.print('<td>');
	  htp.print('<textarea name=i_desc cols=40 rows=5></textarea>');
	  htp.print('</td></tr>');
	  htp.print('</table>');
	  htp.print('</form>');
      htp.formSubmit(cvalue=>'Create New User Profile');
   end;

end; 
/

CREATE OR REPLACE package body forum_users is

   /*
   || Fetch the current user
   */
   function get_current_user return varchar2 is
      cookie owa_cookie.cookie;
	  ret_val varchar2(50) default null;
   begin
      cookie := owa_cookie.get('forum_user');
	  if cookie.num_vals != 0 then
	     ret_val := cookie.vals(1);
	  end if;
	  return ret_val;
   end;

   /*
   || Procedure to set the user using a cookie
   */
   procedure set_user (
      i_username in varchar2 ) is
   begin
      owa_util.mime_header ('text/html',FALSE);
	  owa_cookie.send ('forum_user',i_username);
	  owa_util.http_header_close;
   end;

   /*
   || Procedure to log in user
   */
   function verify_user (
      i_username in varchar2,
	  i_password in varchar2) return boolean is

      match_count number := 0;

   begin
      select count(*) into match_count from members where
	     username = i_username and
		 password = i_password;
      if match_count = 1 then
	     return true;
	  else
	     return false;
      end if;
   end;

   /*
   || Procedure to display a simple login for
   */
   procedure login_form (
      i_username in varchar2 default null,
      i_message in varchar2 default null ) is

   begin
      htp.print('<body bgcolor=white>');
	  if i_message is null then
         htp.print('<h1>Please Log-In!</h1>');
	  else
         htp.print('<h1>' || i_message || '</h1>');
	  end if;
      htp.print('<form action=forum_users.login>');
	  general_form.print_input_row('Forum User Name', 'i_username',FALSE,30,i_username);
	  htp.print ('<p>');
	  general_form.print_input_row('Forum Password', 'i_password', true);
	  htp.print('</table>');
	  htp.formSubmit(cvalue=>'Login');
	  htp.print('</form>');
	  htp.print('<p><p>');
	  htp.anchor('forum_users.create_user_form','Enroll as a new user');
   end;

   /*
   || If the username and password is found, set a cookie and show forums
   || Otherwise, redisplay login form
   */
   procedure login (
      i_username in varchar2,
	  i_password in varchar2) is
   begin
      if verify_user (i_username, i_password) then
	     set_user (i_username);
		 forum.current_forum_list;
	  else
	     login_form (i_username, 'Username/Password not found');
	  end if;
   end;


   /*
   || For to enter user information
   */
   procedure create_user_form is
   begin
      htp.print('<body bgcolor=white>');
      htp.print('<h1>Welcome, New User!</h1><hr>');
      htp.print('<form action=forum_users.save_user_info>');
      -- Existing users cannot change their name or username
	  htp.print('<table>');
	  general_form.print_input_row('Forum User Name', 'i_username');
	  general_form.print_input_row('Real Name', 'i_name');
	  general_form.print_input_row('Forum Password', 'i_password', true);
	  general_form.print_input_row('Email Address', 'i_email_address');
	  general_form.print_textarea_row ('Description','i_desc');
	  htp.print('</table>');
      htp.formSubmit(cvalue=>'Create New User Profile');
	  htp.print('</form>');
   end;

   procedure save_user_info (
      i_username in varchar2 default null,
	  i_name in varchar2 default null,
	  i_password in varchar2 default null,
	  i_email_address in varchar2 default null,
	  i_desc in varchar2 default null ) is
   begin
      -- Create the new user
      insert into members
	     (username, password, name, email_address, personal_desc)
	  values
	    (i_username, i_password, i_name, i_email_address, i_desc );
	  commit;
      set_user (i_username);
	  forum.current_forum_list;
   exception
      when DUP_VAL_ON_INDEX then
	     htp.print('<h1>User already exists!</h1>');
	  when others then
	     htp.print('<h1>An unidentified error occured!</h1>');
   end;

end; 
/
/*--------------------------------------------------------------------------*/

CREATE OR REPLACE package body general_form is

   /*
   || Prints a single two-column table row; column 1 has a description,
   || column 2 has either an input element plain text
   */
   procedure print_input_row (
      i_label in varchar2,
	  i_input_name in varchar2,
	  i_hidden_flag in boolean default false,
	  i_size in number default 40,
	  i_value in varchar2 default null ) is
   begin
      htp.print('<tr>');
	  htp.print('<th align=right>' || i_label ||':</th>');
	  htp.print('<td>');
      if not i_hidden_flag then
         htp.formText ( cname => i_input_name, csize => i_size, cvalue=>i_value );
	  else
         htp.formPassword ( cname => i_input_name, cvalue=>i_value  );
	  end if;
	  htp.print('</td>');
	  htp.print('</tr>');
   end;

   /*
   || Create a two-column table; the second column is a <textarea> field
   */
   procedure print_textarea_row (
      i_label in varchar2,
	  i_input_name in varchar2,
 	  i_cols in number default 40,
 	  i_rows in number default 7 ) is
   begin
	  htp.print('<tr>');
	  htp.print('<th align=right>' || i_label || ':</th>');
	  htp.print('<td>');
	  htp.formTextArea(i_input_name, i_rows, i_cols);
	  htp.print('</td></tr>');
	  htp.print('</table>');
   end;

end; 
/
