CREATE OR REPLACE package xml_invoice_pkg is

   procedure print_invoice (
      i_invoice_number in varchar2 default null );

end;
  
/

CREATE OR REPLACE package body xml_invoice_pkg as


   /*
   || function to return an attribute tag
   */
   function get_attribute (
      attr_name in varchar2,
	  attr_val in varchar2 ) return varchar2 is
   begin
      return attr_name || '=' || '"' || attr_val || '"';
   end;


   /*
   || Simple wrapper procedure to print a tag
   */
   procedure print_xml_tag (
      tag_name in varchar2,
	  tag_value in varchar2,
	  tag_attr in varchar2 default null ) is

      xml_str varchar2(5000);

   begin
      if tag_attr is null then
         xml_str := '<' || tag_name || '>';
	  else
         xml_str := '<' || tag_name || ' ' || tag_attr || ' >';
	  end if;
	  xml_str := xml_str || tag_value;
      xml_str := xml_str || '</' || tag_name || '>';
      htp.print(xml_str);
   end;

   /*
   || Print the items for the selected invoice.  Return
   || the total of the invoice item using an OUT parameter
   */
   procedure print_items (
      i_invoice_id in number,
	  o_invoice_total out number ) is

      cursor item_cur is
         select
		    p.part_num, p.part_name, i.quantity, i.unit_cost
		 from
            xml_invoice_items i,
            xml_parts p
         where
            i.part_id = p.part_id and
			i.invoice_id = i_invoice_id;

	  item_rec item_cur%rowtype;

	  part_num_attr varchar2(500);

   begin
      o_invoice_total := 0;
	  open item_cur;
	  htp.print('<INVOICE_ITEMS>');
	  loop
	     fetch item_cur into item_rec;
		 exit when item_cur%notfound;
		 -- Accumulate costs
		 o_invoice_total := o_invoice_total + item_rec.quantity * item_rec.unit_cost;
		 -- Generate XML tags
		 htp.print('<ITEM>');
		 part_num_attr := get_attribute ('ITEM_NUM',item_rec.part_num);
		 print_xml_tag ('ITEM_NAME', item_rec.part_name, part_num_attr);
         print_xml_tag ('QUANTITY',item_rec.quantity);
         print_xml_tag ('PRICE',item_rec.unit_cost);
		 htp.print('</ITEM>');
      end loop;
	  close item_cur;
	  htp.print ('</INVOICE_ITEMS>');
   end;

   /*
   || Main procedure to print the invoice.
   */
   procedure print_invoice (
      i_invoice_number in varchar2 default null ) is

      cursor inv_cur is
         select
		    i.invoice_id, i.invoice_date, c.customer_name
		 from
		    xml_invoice i,
			xml_customers c
		 where
		    i.customer_id = c.customer_id and
			i.invoice_number = i_invoice_number;

	  inv_rec inv_cur%rowtype;

	  inv_total number default 0;

   begin
      inv_total := 0;
	  open inv_cur;
	  htp.print('<?xml version="1.0"?>');
      -- DTD is omitted in this example
	  htp.print('<!DOCTYPE INVOICE>');
	  htp.print('<INVOICE>');
      fetch inv_cur into inv_rec;
 	  if not inv_cur%notfound then
         print_xml_tag ('INVOICE_NUMBER', i_invoice_number);
         print_xml_tag ('DATE', inv_rec.invoice_date);
         print_xml_tag ('CUSTOMER', inv_rec.customer_name);
		 print_items (inv_rec.invoice_id, inv_total );
         print_xml_tag ('TOTAL', inv_total);
	  end if;
	  close inv_cur;
	  htp.print ('</INVOICE>');
   end;

end;
  
/

